﻿using GiftShopContracts.BindingModels;
using GiftShopContracts.ViewModels;
using GiftShopDataModels.Enums;
using GiftShopDataModels.Models;
using System.ComponentModel.DataAnnotations;

namespace GiftShopDatabaseImplement.Models
{
    public class Order : IOrderModel
    {
        public int Id { get; private set; }

        public int GiftId { get; private set; }

		[Required]
		public int ClientId { get; set; }

        public int? ImplementerId { get; private set; }

        public string GiftName { get; private set; } = string.Empty;

        [Required]
        public int Count { get; private set; }

        [Required]
        public double Sum { get; private set; }

        [Required]
        public OrderStatus Status { get; private set; } = OrderStatus.Неизвестен;

        [Required]
        public DateTime DateCreate { get; private set; } = DateTime.Now;

        public DateTime? DateImplement { get; private set; }

        public virtual Gift Gift { get; set; }

		public Client Client { get; set; }
        public Implementer? Implementer { get; set; }

        public static Order? Create(OrderBindingModel? model, GiftShopDatabase context)
        {
            if (model == null)
            {
                return null;
            }

            return new Order()
            {
                Id = model.Id,
                GiftId = model.GiftId,
				GiftName = model.GiftName,
                Count = model.Count,
                Sum = model.Sum,
                Status = model.Status,
                DateCreate = model.DateCreate,
                DateImplement = model.DateImplement,
				Gift = context.Gifts.FirstOrDefault(x => x.Id == model.GiftId),
				ClientId = model.ClientId,
				Client = context.Clients.FirstOrDefault(x => x.Id == model.ClientId),
				ImplementerId = model.ImplementerId,
				Implementer = (model.ImplementerId.HasValue ? context.Implementers.FirstOrDefault(x => x.Id == model.ImplementerId)
				: null),
			};
        }

        public void Update(OrderBindingModel? model)
        {
            if (model == null)
            {
                return;
            }

            Status = model.Status;
            DateImplement = model.DateImplement;
            ImplementerId = model.ImplementerId;
        }

		public OrderViewModel GetViewModel
		{
			get
			{
				using var context = new GiftShopDatabase();
				return new OrderViewModel
				{
					Id = Id,
					GiftId = GiftId,
					ClientId = ClientId,
                    ImplementerId = ImplementerId,
                    ClientFIO = Client.ClientFIO,
					GiftName = Gift.GiftName,
					Count = Count,
					Sum = Sum,
					Status = Status,
					DateCreate = DateCreate,
					DateImplement = DateImplement,
					ImplementerFIO = (Implementer != null ? Implementer.ImplementerFIO : string.Empty)
				};
			}
		}
	}
}
